# How to Develop and Use Libcaosdb

## Build

We use [cmake](https://cmake.org) as build tool.

1. `mkdir build && cd build/`
2. `conan install .. -s "compiler.libcxx=libstdc++11"`
3. `cmake -B . ..`
4. `cmake --build .`

## Unit Tests

### Build

For the tests there is a slightly different setup required (with option `-D CMAKE_BUILD_TYPE=Debug`)

1. `mkdir build && cd build/`
2. `conan install .. -s "compiler.libcxx=libstdc++11"`
3. `cmake -B . -D CMAKE_BUILD_TYPE=Debug ..`
4. `cmake --build .`

### Run

In the build directory, run `ctest`

### Framework

We use [GoogleTest](https://google.github.io/googletest/) for unit testing.

### Coverage

We use [gcov](https://gcc.gnu.org/onlinedocs/gcc/Gcov.html) and
[lcov](https://github.com/linux-test-project/lcov) for generating test coverage
reports.

In the build directory, generate the coverage report by running
`cmake --build . --target unit_test_coverage`.

Note that this special target will run the tests again. Thus it is not
necessary to run `ctest` prior to this target.

The coverage report can be viewed in a browser by opening
`<build_directory>/unit_test_coverage/index.html`.

## Code Formatting

* install clang-format on your system.
* `clang-format -i --verbose $(find test/ src/ include/ -type f -iname "*.cpp" -o -iname "*.h" -o -iname "*.h.in")`

## Naming Convention

Please adhere to [Google's C++ naming conventions](https://google.github.io/styleguide/cppguide.html#Naming).

## Documentation

In the build directory, run

* `cmake --build . --target doc-doxygen` (generate Doxygen)
* `cmake --build . --target doc-sphinx` (generate Sphinx)
