#include <iostream>
#include <stdio.h>
#include "caosdb/utility.h"
#include "caosdb/constants.h"
#include "caosdb/connection.h"
#include "ccaosdb.h"

extern "C" {

const char *caosdb_utility_get_env_var(const char *name,
                                       const char *fall_back) {
  return caosdb::utility::get_env_var(name, fall_back);
}

int caosdb_connection_create_pem_file_certificate_provider(
  caosdb_connection_certificate_provider *out, const char *path) {
  out->wrapped_certificate_provider =
    new caosdb::connection::PemFileCACertProvider(std::string(path));
  return 0;
}

int caosdb_connection_delete_certificate_provider(
  caosdb_connection_certificate_provider *provider) {
  delete static_cast<caosdb::connection::CACertificateProvider *>(
    provider->wrapped_certificate_provider);
  return 0;
}

int caosdb_authentication_create_plain_password_authenticator(
  caosdb_authentication_authenticator *out, const char *username,
  const char *password) {
  out->wrapped_authenticator =
    new caosdb::authentication::PlainPasswordAuthenticator(
      std::string(username), std::string(password));
  return 0;
}

int caosdb_authentication_delete_authenticator(
  caosdb_authentication_authenticator *authenticator) {
  delete static_cast<caosdb::authentication::Authenticator *>(
    authenticator->wrapped_authenticator);
  return 0;
}

int caosdb_connection_create_tls_connection_configuration(
  caosdb_connection_connection_configuration *out, const char *host,
  const int port, caosdb_authentication_authenticator *authenticator,
  caosdb_connection_certificate_provider *provider) {

  auto host_str = std::string(host);
  if (authenticator != nullptr && provider != nullptr) {
    auto wrapped_provider =
      static_cast<caosdb::connection::CACertificateProvider *>(
        provider->wrapped_certificate_provider);
    auto wrapped_authenticator =
      static_cast<caosdb::authentication::Authenticator *>(
        authenticator->wrapped_authenticator);
    out->wrapped_connection_configuration =
      new caosdb::connection::SslCaosDBConnectionConfig(
        host_str, port, *wrapped_provider, *wrapped_authenticator);
  } else if (authenticator != nullptr) {
    auto wrapped_authenticator =
      static_cast<caosdb::authentication::Authenticator *>(
        authenticator->wrapped_authenticator);
    out->wrapped_connection_configuration =
      new caosdb::connection::SslCaosDBConnectionConfig(host_str, port,
                                                        *wrapped_authenticator);
  } else if (provider != nullptr) {
    auto wrapped_provider =
      static_cast<caosdb::connection::CACertificateProvider *>(
        provider->wrapped_certificate_provider);
    out->wrapped_connection_configuration =
      new caosdb::connection::SslCaosDBConnectionConfig(host_str, port,
                                                        *wrapped_provider);
  } else {
    out->wrapped_connection_configuration =
      new caosdb::connection::SslCaosDBConnectionConfig(host_str, port);
  }
  return 0;
}

int caosdb_connection_create_insecure_connection_configuration(
  caosdb_connection_connection_configuration *out, const char *host,
  const int port) {
  out->wrapped_connection_configuration =
    new caosdb::connection::InsecureCaosDBConnectionConfig(host, port);
  return 0;
}

int caosdb_connection_delete_connection_configuration(
  caosdb_connection_connection_configuration *configuration) {
  delete static_cast<caosdb::connection::CaosDBConnectionConfig *>(
    configuration->wrapped_connection_configuration);
  return 0;
}

int caosdb_connection_create_connection(
  caosdb_connection_connection *out,
  const caosdb_connection_connection_configuration *configuration) {
  caosdb::connection::CaosDBConnectionConfig *config =
    static_cast<caosdb::connection::CaosDBConnectionConfig *>(
      configuration->wrapped_connection_configuration);
  out->wrapped_connection = new caosdb::connection::CaosDBConnection(*config);
  return 0;
}

int caosdb_connection_delete_connection(
  caosdb_connection_connection *connection) {
  delete static_cast<caosdb::connection::CaosDBConnection *>(
    connection->wrapped_connection);
  return 0;
}

int caosdb_connection_get_version_info(
  caosdb_info_version_info *out,
  const caosdb_connection_connection *connection) {
  auto *wrapped_connection =
    static_cast<caosdb::connection::CaosDBConnection *>(
      connection->wrapped_connection);
  auto version_info = wrapped_connection->GetVersionInfo();

  out->major = (int)version_info->GetMajor();
  out->minor = (int)version_info->GetMinor();
  out->patch = (int)version_info->GetPatch();

  // copy pre_release, needs local variable because out->pre_release is const
  char *pre_release =
    (char *)malloc(sizeof(char) * (version_info->GetPreRelease().length() + 1));
  strcpy(pre_release, version_info->GetPreRelease().c_str());
  out->pre_release = pre_release;

  // copy build, needs local variable because out->build is const
  char *build =
    (char *)malloc(sizeof(char) * (version_info->GetBuild().length() + 1));
  strcpy(build, version_info->GetBuild().c_str());
  out->build = build;

  return 0;
}
}
