/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include <gtest/gtest-message.h>   // for Message
#include <gtest/gtest-test-part.h> // for TestPartResult, SuiteApiRes...
#include <memory>                  // for allocator, operator!=, shar...
#include "caosdb/connection.h"     // for PemCACertProvider, Insecure...
#include "gtest/gtest_pred_impl.h" // for Test, AssertionResult, EXPE...

namespace caosdb::connection {

TEST(test_connection, configure_insecure_localhost_8080) {
  InsecureCaosDBConnectionConfig config("localhost", 8000);

  EXPECT_EQ("localhost", config.GetHost());
  EXPECT_EQ(8000, config.GetPort());
  auto icc = config.GetChannelCredentials();
  EXPECT_TRUE(icc != nullptr);
}

TEST(test_connection, configure_ssl_localhost_8080) {
  auto cacert = PemCACertProvider("ca chain");
  SslCaosDBConnectionConfig config("localhost", 44300, cacert);

  EXPECT_EQ("localhost", config.GetHost());
  EXPECT_EQ(44300, config.GetPort());
  auto sslcc = config.GetChannelCredentials();
  EXPECT_TRUE(sslcc != nullptr);
}

} // namespace caosdb::connection
